-- This module provides a decidability predicate. A property "A" is
-- decidable is "A ∨ Not A" holds.

open import Lib.Proposition
open import Lib.Boolean

module Lib.Decidable where

-- ----------------------------------------------------------------------
-- * Definition
  
-- The type "Decidable A" is isomorphic to "A ∨ Not A". It can be used
-- as a type returned by decision procedures. Rather than just
-- deciding whether a proposition is true or false, such a procedure
-- can return a proof of A or a proof of Not A.

data Decidable (A : Set) : Set where
  yes : A -> Decidable A
  no : Not A -> Decidable A

-- ----------------------------------------------------------------------
-- * Operations

-- Extract a boolean from a decidable property.
boolean-of-decidable : ∀ {A} -> Decidable A -> Boolean
boolean-of-decidable (yes _) = true
boolean-of-decidable (no _) = false


-- Forget the proof.
is-yes : ∀ {A} -> Decidable A -> Set
is-yes (yes _) = True
is-yes (no _) = False

is-no : ∀ {A} -> Decidable A -> Set
is-no (yes _) = False
is-no (no _) = True

-- Regain the proof.
yes-witness : ∀ {P}{Q : Decidable P} → is-yes Q → P
yes-witness {Q = yes p} _  = p
yes-witness {Q = no _ } ()

no-witness : ∀ {P}{Q : Decidable P} → is-no Q → Not P
no-witness {Q = yes _} ()
no-witness {Q = no p } _ = p
